/*******************************************************************
    A telegram bot for your ESP8266 that controls the
    onboard LED. The LED in this example is active low.

    Parts:
    D1 Mini ESP8266 * - http://s.click.aliexpress.com/e/uzFUnIe
    (or any ESP8266 board)

      = Affilate

    If you find what I do useful and would like to support me,
    please consider becoming a sponsor on Github
    https://github.com/sponsors/witnessmenow/


    Written by Brian Lough
    YouTube: https://www.youtube.com/brianlough
    Tindie: https://www.tindie.com/stores/brianlough/
    Twitter: https://twitter.com/witnessmenow

    Updated by Riccardo Grosso, 05/31/2024
    https://www.ethicaldiy.org
 *******************************************************************/

#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>
#include <UniversalTelegramBot.h>

// Wifi network station credentials
#define WIFI_SSID "inserisci il nome della tua rete WiFi"
#define WIFI_PASSWORD "inserisci la password della rete WiFi"
// Telegram BOT Token (Get from Botfather)
#define BOT_TOKEN "inserisci il tuo toked (codice alfanumerico lungo)"

const unsigned long BOT_MTBS = 1000; // mean time between scan messages

X509List cert(TELEGRAM_CERTIFICATE_ROOT);
WiFiClientSecure secured_client;
UniversalTelegramBot bot(BOT_TOKEN, secured_client);
unsigned long bot_lasttime; // last time messages' scan has been done

const int led1Pin = 2;
const int led2Pin = 0;
int led1Status = 0;
int led2Status = 0;

/* I led e i relè abitualmente si attivano con ledon e si spengono con ledoff
   Ma in alcuni apparecchi commerciali, i comandi sono invertiti
   se lanciando il programma i led/relè si accendono quando si lancia il comando di spegnimento
   e si spengono quando dovrebbero accendersi, allora commentare le due linee seguenti */
 int changeVal = 1;   //if cangeVal = "1", relay1on = HIGH; relay1off = LOW; else relay1on = LOW; relay1off = HIGH 
 int led1on;
 int led1off;
 int led2on;
 int led2off;

void handleNewMessages(int numNewMessages)
{
  Serial.print("handleNewMessages ");
  Serial.println(numNewMessages);

  for (int i = 0; i < numNewMessages; i++)
  {
    String chat_id = bot.messages[i].chat_id;
    String text = bot.messages[i].text;

    String from_name = bot.messages[i].from_name;
    if (from_name == "")
      //from_name = "Guest";
      from_name = "inserisci il tuo ID (cod. numerico 10 cifre";

    if (text == "led1on" or text == "Led1on" or text == "/led1on" or text == "/Led1on" )
    {
      digitalWrite(led1Pin, led1on); // turn the LED on (HIGH is the voltage level)
      led1Status = 1;
      bot.sendMessage(chat_id, "Il led/relè n. 1 è attivo", "");
    }

    if (text == "led1off" or text == "Led1off" or text == "/led1off" or text == "/Led1off")
    {
      led1Status = 0;
      digitalWrite(led1Pin, led1off); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Il led/relè n. 1 è spento", "");
    }

    if (text == "led2on" or text == "Led2on" or text == "/led2on" or text == "/Led2on")
    {
      digitalWrite(led2Pin, led2on); // turn the LED on (HIGH is the voltage level)
      led2Status = 1;
      bot.sendMessage(chat_id, "Il led/relè n. 2 è attivo", "");
    }

    if (text == "led2off" or text == "Led2off" or text == "/led2off" or text == "/Led2off")
    {
      led2Status = 0;
      digitalWrite(led2Pin, led2off); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Il led/relè n. 1 è spento", "");
    }


    if (text == "ledon" or text == "Ledon" or text == "/ledon" or text == "/Ledon")
    {
      digitalWrite(led1Pin, led1on); // turn the LED on (HIGH is the voltage level)
      led1Status = 1;
      bot.sendMessage(chat_id, "Il led/relè n. 1 è attivo", "");
      digitalWrite(led2Pin, led2on); // turn the LED on (HIGH is the voltage level)
      led2Status = 1;
      bot.sendMessage(chat_id, "Il led/relè n. 2 è attivo", "");
    }

    if (text == "ledoff" or text == "/ledoff" or text == "Ledoff" or text == "/Ledoff")
    {
      //ledStatus = 0;
      digitalWrite(led1Pin, led1off); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Il led/relè n. 1 è spento", "");
      led1Status = 0;
      digitalWrite(led2Pin, led2off); // turn the LED off (LOW is the voltage level)
      bot.sendMessage(chat_id, "Il led/relè n. 2 è spento", "");
      led2Status = 0;
    }


    if (text == "status" or text == "Status" or text == "/status")
    {
      if (led1Status == 1)
      {
        bot.sendMessage(chat_id, "Il led/relè n. 1 è attivo", "");
      }
      else
      {
        bot.sendMessage(chat_id, "Il led/relè n. 1 è spento", "");
      }
      if (led2Status == 1)
      {
        bot.sendMessage(chat_id, "Il led/relè n. 2 è attivo", "");
      }
      else
      {
        bot.sendMessage(chat_id, "Il led/relè n. 2 è spento", "");
      }
    }

    if (text == "/start")
    {
      String welcome = "Benvenuto/a, " + from_name + ".\n";
      welcome += "/ledon :   attiva entrambi i led/relè\n";
      welcome += "/ledoff :  spegne entrambi i led/relè\n";
      welcome += "/led1on :  attiva il led/relè n.1\n";
      welcome += "/led1off : spegne il led/relè n.1\n";
      welcome += "/led2on :  attiva il led/relè n.2\n";
      welcome += "/led2off : spegne il led/relè n.2\n";
      welcome += "/status :  mostra lo stato dei led/relè\n";      
      bot.sendMessage(chat_id, welcome, "Markdown");
    }
  }
}


void setup()
{
  Serial.begin(115200);
  Serial.println();

  if (changeVal == 1){ 
    led1on = 1;      
    led1off = 0;
    led2on = 1;
    led2off = 0;
  }    
  else{                
    led1on = 0;
    led1off = 1;  
    led2on = 0;
    led2off = 1; 
  }
  
  pinMode(led1Pin, OUTPUT); // initialize digital ledPin as an output.
  delay(10);
  digitalWrite(led1Pin, led1off); // initialize pin as off (active LOW)
  pinMode(led2Pin, OUTPUT); // initialize digital ledPin as an output.
  delay(10);
  digitalWrite(led2Pin, led2off); // initialize pin as off (active LOW)
  configTime(0, 0, "pool.ntp.org");      // get UTC time via NTP
  secured_client.setTrustAnchors(&cert); // Add root certificate for api.telegram.org
  Serial.print("Connecting to Wifi SSID ");
  Serial.print(WIFI_SSID);
  WiFi.begin(WIFI_SSID, WIFI_PASSWORD);
  while (WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(500);
  }
  Serial.print("\nWiFi connected. IP address: ");
  Serial.println(WiFi.localIP());

  // Check NTP/Time, usually it is instantaneous and you can delete the code below.
  Serial.print("Retrieving time: ");
  time_t now = time(nullptr);
  while (now < 24 * 3600)
  {
    Serial.print(".");
    delay(100);
    now = time(nullptr);
  }
  Serial.println(now);
}

void loop()
{
  if (millis() - bot_lasttime > BOT_MTBS)
  {
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    while (numNewMessages)
    {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
      numNewMessages = bot.getUpdates(bot.last_message_received + 1);
    }

    bot_lasttime = millis();
  }
}
